from tkinter import *
from tkinter.font import Font as tkfont
import sys

global cfp
if str.upper(sys.platform)=="WIN32":
    cfp = 'win'
else:
    cfp = 'rasp'

class fUtil():
    def __init__(self):
        if cfp == 'win':
            self.defont = "Arial"
        else: #Liberation Sans is the Linux Available version of Arial
            self.defont = "LiberationSans"

    def mkfont(size:int,bold:bool=False,font:str="default"):
        """Creates a font string with wom fonts"""

        if font == "default":
            return f"{fUtil().defont} {size}{' bold' if bold is True else ''}"
        
    def autosize(e:Event=None, relations:list[Widget]=None, offset:int=None,shrink:int=0):
        """Automatically sizes text given a widget with text."""
        if not e is None: #Always need an event
            #Get info from widget
            widget:Widget = e.widget
            w = widget.winfo_width() - 1 - shrink
            h = widget.winfo_height() - 1
            if w < 1 or h < 1:
                return None
            _root=widget.winfo_toplevel()
            ffont = tkfont(_root, widget["font"])
            fw = ffont.measure(widget["text"])
            fh = ffont.metrics('linespace')

            #Check widget with tightest relation
            if not relations is None:
                tempwi = widget
                for wi in relations:
                    if isinstance(wi["text"],str):
                        if wi.winfo_width() > 1:
                            test = (wi.winfo_width() - 1 - ffont.measure(wi["text"]) < w-fw)
                        else:
                            test = False
                    else: test = False
                    if test:
                        widget = wi
                    else:
                        if wi.winfo_height() - 1 < h:#text has same height so only widget height must be smaller
                            if wi.winfo_height() > 1:
                                widget = wi
                    w = widget.winfo_width() - 1 - shrink
                    h = widget.winfo_height() - 1
                    fw = ffont.measure(widget["text"])
                relations.append(tempwi)

            #Final Control Sizes
            _family = ffont.actual(option="family").replace(" ", "")
            _size = ffont.actual(option="size")
            _weight = ffont.actual(option="weight")

            while True: #Make text larger than frame
                if fw < w and fh < h:
                    _size = _size + 1
                else:
                    break
                ffont = tkfont(widget.winfo_toplevel(), f"{_family} {_size} {_weight}")
                fh = ffont.metrics('linespace')
                fw = ffont.measure(widget["text"])

            while True: #Make text fit in frame
                if fw <= w and fh <= h:
                    break
                else:
                    if _size == 0:break
                    _size = _size - 1
                ffont = tkfont(_root, f"{_family} {_size} {_weight}")
                fh = ffont.metrics('linespace')
                fw = ffont.measure(widget["text"])

            if not offset is None: #Apply offset
                _size = _size - offset

            if not relations is None: #Correct relations
                for wi in relations:
                    wi.configure(font = f"{_family} {_size} {_weight}")
            else:
                widget.configure(font = f"{_family} {_size} {_weight}")



from tkinter import *
from tkinter import ttk
import subprocess
import os

class MenuItem():
    """Each item in the menu is created from the corresponding .json file. Each path should be given relative to xyz/WOM/
    """
    def __init__(self,parent:Frame|Toplevel|LabelFrame|Tk,path,nav,*args,**kwargs):
        """Create an item in a row on the menu
        Args:
            root (Tk): Master root for destruction on redirect
            _root (Toplevel): Toplevel object to create menu items in
            path (str): Name of .exe or absolute path to python script
            nav (str): Navigation character to click button
        """
        self.button = Button(master=parent, *args, **kwargs)
        self.parent = parent
        self.root = parent.winfo_toplevel()
        self.path = path
        self.nav = nav
        self.button.config(command = self.itemPath)
        enter = lambda event: event.widget.configure(background="dodger blue")
        leave = lambda event: event.widget.configure(background="gray94")
        self.button.bind("<Enter>", enter)
        self.button.bind("<Leave>", leave)
        #self.button.pack()

    def itemPath(self):
        """Opens the given path or exe for the button
        """
        if ".exe" in self.path:
            os.startfile(self.path)
        else:
            subprocess.call("pythonw.exe "+self.path)
        self.root.destroy()